CONTEXT = "context"
PROMPT = "prompt"
RESPONSE = "response"
TRACE_ID = "trace_id"
PARENT_ID = "parent_id"
TRACE_URI = "trace_uri"
OPEN_INFERENCE_SPAN_KIND = "openinference.span.kind"
ATTRIBUTES = "attributes"
CHAIN = "CHAIN"
INPUT_VALUE = "input.value"
OUTPUT_VALUE = "output.value"
RETRIEVER = "RETRIEVER"
TRACE = "trace"
APPLICATION_JSON = "application/json"
DATA = "data"
UPLOAD_PRE_SIGNED_URL = "uploadPresignedUrl"
FILE_PATH = "filePath"
GET_PRE_SIGNED_URL_API = "/api/project/upload/preSignedUrls/"
